# Django Imports
from django import forms

# NetBox Imports
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from ipam.models import IPAddress

# Proxbox Imports
from ..models import FastAPIEndpoint


class FastAPIEndpointForm(NetBoxModelForm):
    """
    Form for FastAPIEndpoint model.
    It is used to CREATE and UPDATE FastAPIEndpoint objects.
    """

    ip_address = DynamicModelChoiceField(
        queryset=IPAddress.objects.all()
    )

    comments = CommentField()

    class Meta:
        model = FastAPIEndpoint
        fields = (
            'name', 'ip_address', 'port',
            'verify_ssl', 'tags'
        )


class FastAPIEndpointFilterForm(NetBoxModelFilterSetForm):
    """
    Filter form for FastAPIEndpoint model.
    It is used in the FastAPIEndpointListView.
    """

    model = FastAPIEndpoint
    name = forms.CharField(
        required=False
    )
    ip_address = forms.ModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False,
        help_text='Select IP Address'
    )


from netbox.api.viewsets import NetBoxModelViewSet

from .. import filtersets, models
from .serializers import (
    ProxmoxEndpointSerializer,
    NetBoxEndpointSerializer,
    FastAPIEndpointSerializer,
)


class ProxmoxEndpointViewSet(NetBoxModelViewSet):
    queryset = models.ProxmoxEndpoint.objects.all()
    serializer_class = ProxmoxEndpointSerializer


class NetBoxEndpointViewSet(NetBoxModelViewSet):
    queryset = models.NetBoxEndpoint.objects.all()
    serializer_class = NetBoxEndpointSerializer


class FastAPIEndpointViewSet(NetBoxModelViewSet):
    queryset = models.FastAPIEndpoint.objects.all()
    serializer_class = FastAPIEndpointSerializer

    


from django.db.models import Q
from netbox.filtersets import NetBoxModelFilterSet
from .models import ProxmoxEndpoint, NetBoxEndpoint, FastAPIEndpoint


class ProxmoxEndpointFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = ProxmoxEndpoint
        fields = ['id', 'name', 'ip_address', 'mode']
    
    def search(self, queryset, name, value):
            return queryset.filter(name__icontains=value)


class NetBoxEndpointFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = NetBoxEndpoint
        fields = ['id', 'name', 'ip_address']

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class FastAPIEndpointFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = FastAPIEndpoint
        fields = ['id', 'name', 'ip_address']
    
    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)



import django_tables2 as tables
from netbox.tables import NetBoxTable, ChoiceFieldColumn
from netbox.tables.columns import BooleanColumn

from .models import (
    ProxmoxEndpoint,
    NetBoxEndpoint,
    FastAPIEndpoint,
)

class ProxmoxEndpointTable(NetBoxTable):
    name = tables.Column(linkify=True)
    ip_address = tables.Column(linkify=True)
    mode = ChoiceFieldColumn()
    verify_ssl = BooleanColumn()
    
    class Meta(NetBoxTable.Meta):
        model = ProxmoxEndpoint
        fields = (
            'pk', 'id', 'name', 'ip_address', 'port',
            'mode', 'version', 'repoid', 'username', 'token_name',
            'verify_ssl', 'actions',
        )
        
        default_columns = (
            'pk', 'name', 'ip_address', 'port', 'mode',
            'version', 'verify_ssl'
        )


class NetBoxEndpointTable(NetBoxTable):
    name = tables.Column(linkify=True)
    ip_address = tables.Column(linkify=True)
    verify_ssl = BooleanColumn()
    
    class Meta(NetBoxTable.Meta):
        model = NetBoxEndpoint
        fields = (
            'pk', 'id', 'name', 'ip_address', 'port',
            'verify_ssl', 'actions',
        )
        
        default_columns = (
            'pk', 'name', 'ip_address', 'port', 'verify_ssl'
        )


class FastAPIEndpointTable(NetBoxTable):
    name = tables.Column(linkify=True)
    ip_address = tables.Column(linkify=True)
    verify_ssl = BooleanColumn()
    
    class Meta(NetBoxTable.Meta):
        model = FastAPIEndpoint
        fields = (
            'pk', 'id', 'name', 'ip_address', 'port',
            'verify_ssl', 'actions',
        )
        
        default_columns = (
            'pk', 'name', 'ip_address', 'port', 'verify_ssl',
        )


# Django Imports
from django.http import HttpRequest, HttpResponse
from django.shortcuts import render
from django.views.decorators.http import require_GET
from django.urls import reverse
import requests

# Django-HTMX Imports
from django_htmx.middleware import HtmxDetails
from django_htmx.http import replace_url

class HtmxHttpRequest(HttpRequest):
    htmx: HtmxDetails

from netbox_proxbox.models import *

@require_GET
def get_service_status(
    request: HtmxHttpRequest,
    service: str,
    pk: int,
) -> HttpResponse:
    """Get the status of a service."""
    template_name: str = 'netbox_proxbox/status_badge.html'
    
    # Accept only HTMX requests to render this view.
    if not request.htmx:
        return HttpResponse(status=400)
    
    host: str = ''
    url: str = ''
    status: str = 'unknown'
    
    if service == 'fastapi':
        fastapi_service_obj = FastAPIEndpoint.objects.get(pk=pk)
    else:
        fastapi_service_obj = FastAPIEndpoint.objects.first()
    
    if fastapi_service_obj:
        host = str(fastapi_service_obj.ip_address.address).split('/')[0]
        url: str = f"http://{host}:{fastapi_service_obj.port}"
    
    if service == 'proxmox':
        proxmox_service_obj = ProxmoxEndpoint.objects.get(pk=pk)
            
        if proxmox_service_obj:
            proxmox_host: str = str(proxmox_service_obj.ip_address).split('/')[0]
            url = f'{url}/proxmox/version?ip_address={proxmox_host}&port={proxmox_service_obj.port}'
        
    if service == 'netbox':
        netbox_service_obj = NetBoxEndpoint.objects.get(pk=pk)
        netbox_ip = str(netbox_service_obj.ip_address.address).split('/')[0]
        
        netbox_endpoint_url = f'{url}/netbox/endpoint'
        
        try:
            # Check if NetBoxEndpoint exists on FastAPI database.
            response = requests.get(netbox_endpoint_url)
            response.raise_for_status()
            response = list(response.json())
            
            netbox = None
            
            if len(response) > 0:
                for nb in response:
                    if (nb['id'] == pk) and (nb['ip_address'] == netbox_ip):
                        netbox = nb
                        break
                
                if netbox:
                    # Delete all NetBoxEndpoints from FastAPI database, except the one that matches the current NetBoxEndpoint.
                    for nb in response:
                        if nb['id'] != pk:
                            requests.delete(f'{netbox_endpoint_url}/{nb["id"]}')
                else:
                    # Delete all NetBoxEndpoints from FastAPI database.
                    for nb in response:
                        requests.delete(f'{netbox_endpoint_url}/{nb["id"]}')
            
            if netbox:
                # NetBoxEndpoint exists on FastAPI database. Check if it is alive.
                url = f'{url}/netbox/status'
            else:
                # Create NetBoxEndpoint on FastAPI database.
                requests.post(netbox_endpoint_url, json={
                    'id': pk,
                    'name': netbox_service_obj.name,
                    'ip_address': netbox_ip,
                    'port': netbox_service_obj.port,
                    'token': netbox_service_obj.token,
                    'verify_ssl': netbox_service_obj.verify_ssl,
                })
            
        except requests.exceptions.HTTPError as err:
            print(f'HTTP error ocurred: {err}')
            status = 'error'
        
        except Exception as errr:
            print(f'Error ocurred: {errr}')
            status = 'error'
            
        
    try:
        response = requests.get(url)
        response.raise_for_status()
        status = 'success'
        
    except requests.exceptions.HTTPError as err:
        print(f'HTTP error ocurred: {err}')
        status = 'error'
        
    except Exception as errr:
        print(f'Error ocurred: {errr}')
        status = 'error'
    
    return render(
        request,
        template_name,
        {
            'status': status
        }
    )
 


from django.urls import reverse
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.core.validators import MaxValueValidator, MinValueValidator 

from netbox.models import NetBoxModel

from .choices import ProxmoxModeChoices

class ProxmoxEndpoint(NetBoxModel):
    name = models.CharField(
        default='Proxmox Endpoint',
        max_length=255,
        blank=True,
        null=True,
        help_text=_('Name of the Proxmox Endpoint/Cluster. It will be filled automatically by API.'),
    )
    ip_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('IP Address'),
        null=True,
        help_text=_('IP Address of the Proxmox Endpoint (Cluster). It will try using the DNS name provided in IP Address if it is not empty.'),
    )
    port = models.PositiveIntegerField(
        default=8006,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('HTTP Port'),
    )
    mode = models.CharField(
        max_length=255,
        choices=ProxmoxModeChoices,
        default=ProxmoxModeChoices.PROXMOX_MODE_UNDEFINED,

    )
    version = models.CharField(max_length=20, blank=True, null=True)
    repoid = models.CharField(
        max_length=16,
        blank=True,
        null=True,
        verbose_name=_('Repository ID'),
    )
    username = models.CharField(
        default='root@pam',
        max_length=255,
        verbose_name=_('Username'),
        help_text=_("Username must be in the format of 'user@realm'. Default is 'root@pam'.")
    )
    password = models.CharField(
        max_length=255,
        verbose_name=_('Password'),
        help_text=_('Password of the Proxmox Endpoint. It is not needed if you use Token.'),
        blank=True,
        null=True,
    )
    token_name = models.CharField(
        max_length=255,
        verbose_name=_('Token Name'),
    )
    token_value = models.CharField(
        max_length=255,
        verbose_name=_('Token Value'),
    )
    verify_ssl = models.BooleanField(
        default=True,
        verbose_name=_('Verify SSL'),
        help_text=_('Choose or not to verify SSL certificate of the Proxmox Endpoint'),
    )

    class Meta:
        verbose_name_plural: str = "Proxmox Endpoints"
        unique_together = ['name', 'ip_address']
        ordering = ('name',)
        
    def __str__(self):
        return f"{self.name} ({self.ip_address})"
    
    def get_absolute_url(self):
        return reverse('plugins:netbox_proxbox:proxmoxendpoint', args=[self.pk])


class NetBoxEndpoint(NetBoxModel):
    name = models.CharField(
        default='NetBox Endpoint',
        max_length=255,
        blank=True,
        null=True,
        help_text=_('Name of the NetBox Endpoint.'),
    )
    ip_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('IP Address'),
        null=True,
        help_text=_('IP Address of the NetBox. It will try using the DNS name provided in IP Address if it is not empty.'),
    )
    port = models.PositiveIntegerField(
        default=443,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('HTTP Port'),
    )
    token = models.CharField(max_length=255)
    verify_ssl = models.BooleanField(
        default=True,
        verbose_name=_('Verify SSL'),
        help_text=_('Choose or not to verify SSL certificate of the Netbox Endpoint'),
    )

    class Meta:
        verbose_name_plural: str = 'Netbox Endpoints'
        unique_together = ['name', 'ip_address']
        
    def __str__(self):
        return f"{self.name} ({self.ip_address})"

    def get_absolute_url(self):
        return reverse("plugins:netbox_proxbox:netboxendpoint", args=[self.pk])
        

class FastAPIEndpoint(NetBoxModel):
    name = models.CharField(
        default='ProxBox Endpoint',
        max_length=255,
        blank=True,
        null=True,
        help_text=_('Name of the ProxBox Endpoint.'),
    )
    ip_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('IP Address'),
        null=True,
        help_text=_('IP Address of the Proxbox API (Backend Service). It will try using the DNS name provided in IP Address if it is not empty.'),
    )
    port = models.PositiveIntegerField(
        default=8800,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('HTTP Port'),
    )
    verify_ssl = models.BooleanField(default=True)

    class Meta:
        verbose_name_plural: str = 'FastAPI Endpoints'
        unique_together = ['name', 'ip_address']
    
    def __str__(self):
        return f"{self.name} ({self.ip_address})"

    def get_absolute_url(self):
        return reverse("plugins:netbox_proxbox:fastapiendpoint", args=[self.pk])


from django.utils.translation import gettext_lazy as _
from utilities.choices import ChoiceSet

class ProxmoxModeChoices(ChoiceSet):
    key = 'ProxmoxEndpoint.mode'
    
    PROXMOX_MODE_UNDEFINED = 'undefined'
    PROXMOX_MODE_STANDALONE = 'standalone'
    PROXMOX_MODE_CLUSTER = 'cluster'
    
    CHOICES = [
        (PROXMOX_MODE_UNDEFINED, _('Undefined'), 'gray'),
        (PROXMOX_MODE_STANDALONE, _('Standalone'), 'blue'),
        (PROXMOX_MODE_CLUSTER, _('Cluster'), 'green'),
    ]


from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from ipam.api.serializers import IPAddressSerializer
from ..models import ProxmoxEndpoint, NetBoxEndpoint, FastAPIEndpoint


class ProxmoxEndpointSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:proxmoxendpoint-detail',
    )
    ip_address = IPAddressSerializer()
    
    class Meta:
        model = ProxmoxEndpoint
        fields = (
            'id', 'url', 'display', 'name', 'ip_address', 'port',
            'token_name', 'token_value', 'username', 'password', 'verify_ssl',
            'mode', 'version', 'repoid', 
            'tags', 'custom_fields', 'created', 'last_updated',
        )


class NetBoxEndpointSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:netboxendpoint-detail',
    )
    ip_address = IPAddressSerializer()
    
    class Meta:
        model = NetBoxEndpoint
        fields = (
            'id', 'url', 'display', 'name', 'ip_address', 'port',
            'token', 'verify_ssl', 'tags', 'custom_fields',
            'created', 'last_updated',
        )


class FastAPIEndpointSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:fastapiendpoint-detail',
    )
    ip_address = IPAddressSerializer()
    
    class Meta:
        model = FastAPIEndpoint
        fields = (
            'id', 'url', 'display', 'name', 'ip_address', 'port',
            'verify_ssl', 'tags', 'custom_fields', 'created', 'last_updated',
        )



# Django Imports
from django import forms

# NetBox Imports
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, DynamicModelChoiceField
from ipam.models import IPAddress

# Proxbox Imports
from ..models import ProxmoxEndpoint
from ..choices import ProxmoxModeChoices


class ProxmoxEndpointForm(NetBoxModelForm):
    """
    Form for ProxmoxEndpoint model.
    It is used to CREATE and UPDATE ProxmoxEndpoint objects.
    """
    ip_address = DynamicModelChoiceField(
        queryset=IPAddress.objects.all()
    )
    
    comments = CommentField()
    
    class Meta:
        model = ProxmoxEndpoint
        fields = (
            'name', 'ip_address', 'port', 'username',
            'password', 'token_name', 'token_value', 'verify_ssl',
            'tags'
        )


class ProxmoxEndpointFilterForm(NetBoxModelFilterSetForm):
    """
    Filter form for ProxmoxEndpoint model.
    It is used in the ProxmoxEndpointListView.
    """
    
    model = ProxmoxEndpoint
    name = forms.CharField(
        required=False
    )
    ip_address = forms.ModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False,
        help_text='Select IP Address'
    )
    mode = forms.MultipleChoiceField(
        choices=ProxmoxModeChoices,
        required=False
    )
